@@ -1,5 +1,5 @@
 /* source: xio-socket.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger 2001-2009 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for socket related functions, and the
@@ -196,6 +196,8 @@ const struct optdesc opt_setsockopt_int    = { "setsockopt-int",    "sockopt-int
 const struct optdesc opt_setsockopt_bin    = { "setsockopt-bin",    "sockopt-bin",    OPT_SETSOCKOPT_BIN,        GROUP_SOCKET,PH_PASTSOCKET,TYPE_INT_INT_BIN,     OFUNC_SOCKOPT_GENERIC, 0, 0 };
 const struct optdesc opt_setsockopt_string = { "setsockopt-string", "sockopt-string", OPT_SETSOCKOPT_STRING,     GROUP_SOCKET,PH_PASTSOCKET,TYPE_INT_INT_STRING,  OFUNC_SOCKOPT_GENERIC, 0, 0 };
 
+const struct optdesc opt_null_eof = { "null-eof", NULL, OPT_NULL_EOF, GROUP_SOCKET, PH_INIT, TYPE_BOOL, OFUNC_OFFSET, (bool)&((xiofile_t *)0)->stream.para.socket.null_eof };
+
 
 #if WITH_GENERICSOCKET
 
@@ -212,7 +214,7 @@ int xioopen_socket_connect(int argc, const char *argv[], struct opt *opts,
    int proto;
    int socktype = SOCK_STREAM;
    int needbind = 0;
-   union sockaddr_union them;  socklen_t themlen;
+   union sockaddr_union them;  socklen_t themlen; size_t themsize;
    union sockaddr_union us;    socklen_t uslen = sizeof(us);
    int result;
 
@@ -237,20 +239,20 @@ int xioopen_socket_connect(int argc, const char *argv[], struct opt *opts,
    /*retropt_int(opts, OPT_IP_PROTOCOL, &proto);*/
    xfd->howtoend = END_SHUTDOWN;
 
-   applyopts(-1, opts, PH_INIT);
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
+   applyopts(-1, opts, PH_INIT);
    applyopts(-1, opts, PH_EARLY);
 
-   themlen = 0;
+   themsize = 0;
    if ((result =
-	dalan(address, (char *)&them.soa.sa_data, &themlen, sizeof(them)))
+	dalan(address, (char *)&them.soa.sa_data, &themsize, sizeof(them)))
        < 0) {
       Error1("data too long: \"%s\"", address);
    } else if (result > 0) {
       Error1("syntax error in \"%s\"", address);
    }      
    them.soa.sa_family = pf;
-   themlen +=
+   themlen = themsize +
 #if HAVE_STRUCT_SOCKADDR_SALEN
       sizeof(them.soa.sa_len) +
 #endif
@@ -292,7 +294,7 @@ int xioopen_socket_listen(int argc, const char *argv[], struct opt *opts,
    int pf;
    int proto;
    int socktype = SOCK_STREAM;
-   union sockaddr_union us;  socklen_t uslen;
+   union sockaddr_union us;  socklen_t uslen; size_t ussize;
    struct opt *opts0;
    int result;
 
@@ -318,15 +320,15 @@ int xioopen_socket_listen(int argc, const char *argv[], struct opt *opts,
    xfd->howtoend = END_SHUTDOWN;
 
    socket_init(0, &us);
-   uslen = 0;
+   ussize = 0;
    if ((result =
-	dalan(usname, (char *)&us.soa.sa_data, &uslen, sizeof(us)))
+	dalan(usname, (char *)&us.soa.sa_data, &ussize, sizeof(us)))
        < 0) {
       Error1("data too long: \"%s\"", usname);
    } else if (result > 0) {
       Error1("syntax error in \"%s\"", usname);
    }
-   uslen += sizeof(us.soa.sa_family)
+   uslen = ussize + sizeof(us.soa.sa_family)
 #if HAVE_STRUCT_SOCKADDR_SALEN
       + sizeof(us.soa.sa_len)
 #endif
@@ -379,8 +381,8 @@ int _xioopen_socket_sendto(const char *pfname, const char *type,
    xiosingle_t *xfd = &xxfd->stream;
    char *garbage;
    union sockaddr_union us = {{0}};
-   socklen_t uslen = 0;
-   socklen_t themlen = 0;
+   socklen_t uslen = 0;   size_t ussize;
+   size_t themsize;
    int pf;
    int socktype = SOCK_RAW;
    int proto;
@@ -409,16 +411,16 @@ int _xioopen_socket_sendto(const char *pfname, const char *type,
    xfd->howtoend = END_SHUTDOWN;
 
    xfd->peersa.soa.sa_family = pf;
-   themlen = 0;
+   themsize = 0;
    if ((result =
-	dalan(address, (char *)&xfd->peersa.soa.sa_data, &themlen,
+	dalan(address, (char *)&xfd->peersa.soa.sa_data, &themsize,
 	      sizeof(xfd->peersa)))
        < 0) {
       Error1("data too long: \"%s\"", address);
    } else if (result > 0) {
       Error1("syntax error in \"%s\"", address);
-   }      
-   xfd->salen = themlen + sizeof(sa_family_t)
+   }
+   xfd->salen = themsize + sizeof(sa_family_t)
 #if HAVE_STRUCT_SOCKADDR_SALEN
       + sizeof(xfd->peersa.soa.sa_len)
 #endif
@@ -426,7 +428,7 @@ int _xioopen_socket_sendto(const char *pfname, const char *type,
 #if HAVE_STRUCT_SOCKADDR_SALEN
    xfd->peersa.soa.sa_len =
       sizeof(xfd->peersa.soa.sa_len) + sizeof(xfd->peersa.soa.sa_family) +
-      themlen;
+      themsize;
 #endif
 
    /* ...res_opts[] */
@@ -440,16 +442,16 @@ int _xioopen_socket_sendto(const char *pfname, const char *type,
    xfd->dtype = XIODATA_RECVFROM;
 
    if (retropt_string(opts, OPT_BIND, &bindstring) == 0) {
-      uslen = 0;
+      ussize = 0;
       if ((result =
-	   dalan(bindstring, (char *)&us.soa.sa_data, &uslen, sizeof(us)))
+	   dalan(bindstring, (char *)&us.soa.sa_data, &ussize, sizeof(us)))
 	  < 0) {
 	 Error1("data too long: \"%s\"", bindstring);
       } else if (result > 0) {
 	 Error1("syntax error in \"%s\"", bindstring);
       }
       us.soa.sa_family = pf;
-      uslen += sizeof(sa_family_t)
+      uslen = ussize + sizeof(sa_family_t)
 #if HAVE_STRUCT_SOCKADDR_SALEN
 	 + sizeof(us.soa.sa_len)
 #endif
@@ -475,7 +477,7 @@ int xioopen_socket_recvfrom(int argc, const char *argv[], struct opt *opts,
    const char *address = argv[4];
    char *garbage;
    union sockaddr_union *us = &xfd->para.socket.la;
-   socklen_t uslen = sizeof(*us);
+   socklen_t uslen; size_t ussize;
    int pf, socktype, proto;
    char *rangename;
    int result;
@@ -506,16 +508,16 @@ int xioopen_socket_recvfrom(int argc, const char *argv[], struct opt *opts,
    /*retropt_int(opts, OPT_IP_PROTOCOL, &proto);*/
    xfd->howtoend = END_NONE;
 
-   uslen = 0;
+   ussize = 0;
    if ((result =
-	dalan(address, (char *)&us->soa.sa_data, &uslen, sizeof(*us)))
+	dalan(address, (char *)&us->soa.sa_data, &ussize, sizeof(*us)))
        < 0) {
       Error1("data too long: \"%s\"", address);
    } else if (result > 0) {
       Error1("syntax error in \"%s\"", address);
    }      
    us->soa.sa_family = pf;
-   uslen += sizeof(us->soa.sa_family)
+   uslen = ussize + sizeof(us->soa.sa_family)
 #if HAVE_STRUCT_SOCKADDR_SALEN
       + sizeof(us->soa.sa_len);
 #endif
@@ -552,7 +554,7 @@ int xioopen_socket_recv(int argc, const char *argv[], struct opt *opts,
    const char *address = argv[4];
    char *garbage;
    union sockaddr_union us;
-   socklen_t uslen = sizeof(us);
+   socklen_t uslen; size_t ussize;
    int pf, socktype, proto;
    char *rangename;
    int result;
@@ -583,16 +585,16 @@ int xioopen_socket_recv(int argc, const char *argv[], struct opt *opts,
    /*retropt_int(opts, OPT_IP_PROTOCOL, &proto);*/
    xfd->howtoend = END_NONE;
 
-   uslen = 0;
+   ussize = 0;
    if ((result =
-	dalan(address, (char *)&us.soa.sa_data, &uslen, sizeof(us)))
+	dalan(address, (char *)&us.soa.sa_data, &ussize, sizeof(us)))
        < 0) {
       Error1("data too long: \"%s\"", address);
    } else if (result > 0) {
       Error1("syntax error in \"%s\"", address);
    }      
    us.soa.sa_family = pf;
-   uslen += sizeof(sa_family_t)
+   uslen = ussize + sizeof(sa_family_t)
 #if HAVE_STRUCT_SOCKADDR_SALEN
       +sizeof(us.soa.sa_len)
 #endif
@@ -631,7 +633,7 @@ int xioopen_socket_datagram(int argc, const char *argv[], struct opt *opts,
    const char *address = argv[4];
    char *garbage;
    char *rangename;
-   socklen_t themlen;
+   size_t themsize;
    int pf;
    int result;
 
@@ -651,20 +653,20 @@ int xioopen_socket_datagram(int argc, const char *argv[], struct opt *opts,
    xfd->howtoend = END_SHUTDOWN;
 
    xfd->peersa.soa.sa_family = pf;
-   themlen = 0;
+   themsize = 0;
    if ((result =
-	dalan(address, (char *)&xfd->peersa.soa.sa_data, &themlen,
+	dalan(address, (char *)&xfd->peersa.soa.sa_data, &themsize,
 	      sizeof(xfd->peersa)))
        < 0) {
       Error1("data too long: \"%s\"", address);
    } else if (result > 0) {
       Error1("syntax error in \"%s\"", address);
    }
-   xfd->salen = themlen + sizeof(sa_family_t);
+   xfd->salen = themsize + sizeof(sa_family_t);
 #if HAVE_STRUCT_SOCKADDR_SALEN
    xfd->peersa.soa.sa_len =
       sizeof(xfd->peersa.soa.sa_len) + sizeof(xfd->peersa.soa.sa_family) +
-      themlen;
+      themsize;
 #endif
 
    if ((result =
@@ -849,10 +851,10 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 		  themlen, strerror(errno));
 	    timeout = xfd->para.socket.connect_timeout;
 	    writefd.fd = xfd->fd;
-	    writefd.events = (POLLIN|POLLHUP|POLLERR);
+	    writefd.events = (POLLOUT|POLLERR);
 	    result = xiopoll(&writefd, 1, &timeout);
 	    if (result < 0) {
-	       Msg4(level, "xiopoll({%d,POLLIN|POLLHUP|POLLER},,{"F_tv_sec"."F_tv_usec"): %s",
+	       Msg4(level, "xiopoll({%d,POLLOUT|POLLERR},,{"F_tv_sec"."F_tv_usec"): %s",
 		    xfd->fd, timeout.tv_sec, timeout.tv_usec, strerror(errno));
 	       return STAT_RETRYLATER;
 	    }
@@ -862,7 +864,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 		    strerror(ETIMEDOUT));
 	       return STAT_RETRYLATER;
 	    }
-	    if (writefd.revents & POLLOUT) {
+	    if (writefd.revents & POLLERR) {
 #if 0
 	       unsigned char dummy[1];
 	       Read(xfd->fd, &dummy, 1);	/* get error message */
