@@ -1,5 +1,5 @@
 /* source: xio.h */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger 2001-2009 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_h_included
@@ -146,6 +146,13 @@ typedef struct single {
    bool   opt_unlink_close;	/* option unlink_close */
    char  *unlink_close;	/* name of a symlink or unix socket to be removed */
    int dtype;
+   enum {
+      XIOSHUT_UNSPEC,	/* standard (address dependent) behaviour */
+      XIOSHUT_NONE,	/* do nothing on shutdown */
+      XIOSHUT_CLOSE,	/* close the FD */
+      XIOSHUT_DOWN,	/* shutdown() */
+      XIOSHUT_NULL	/* send an empty packet (dgram socket) */
+   } howtoshut;
    enum {
       END_UNSPEC,	/* after init, when no end-close... option */
       END_NONE,		/* no action */
@@ -177,7 +184,7 @@ typedef struct single {
       struct {
 	 struct timeval connect_timeout; /* how long to hang in connect() */
 	 union sockaddr_union la;	/* local socket address */
-	 bool emptyiseof;	/* with dgram: empty packet means EOF */
+	 bool null_eof;		/* with dgram: empty packet means EOF */
 	 bool dorange;
 	 struct xiorange range;	/* restrictions for peer address */
 #if _WITH_IP4 || _WITH_IP6
@@ -340,7 +347,16 @@ union integral {
 } ;
 
 /* some aliases */
-#define u_off u_long	/* please report when this causes problems */
+
+#if HAVE_BASIC_OFF_T==3
+#  define u_off u_int
+#elif HAVE_BASIC_OFF_T==5
+#  define u_off u_long
+#elif HAVE_BASIC_OFF_T==7
+#  define u_off u_longlong
+#else
+#  error "unexpected size of off_t, please report this as bug"
+#endif
 
 #if defined(HAVE_BASIC_OFF64_T) && HAVE_BASIC_OFF64_T
 #  if HAVE_BASIC_OFF64_T==5
